/*
    AutoX11, a tool to automate tasks in an X11 environment
    Copyright (C) 2014  Alicia <alicia@ion.nu>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <string.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include <gdk/gdkx.h>
#include "grab.h"
#include "autox11.h"
#include "designer.h"

void add_imgcmd(GtkButton* unused, void* cmdopts)
{
  struct imgopts* opts=cmdopts;
  while(gtk_events_pending()){gtk_main_iteration();}
  sleep(opts->delay);
  GdkWindow* gdkwindow=gtk_widget_get_parent_window(srcview);
  Display* display=GDK_WINDOW_XDISPLAY(gdkwindow);
  struct rect target;
  if(grabrect(display, &target))
  {
    // Popup error message if we failed to grab the mouse
    GtkWidget* dialog=gtk_message_dialog_new(GTK_WINDOW(mainwindow), GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, "Failed to grab mouse");
    gtk_dialog_run(GTK_DIALOG(dialog));
    gtk_widget_destroy(dialog);
    return;
  }
  if(!target.w){target.w=1;}
  if(!target.h){target.h=1;}
  XImage* screen=getscreenshot(display, XDefaultScreen(display));
  // Create a gdk pixbuf
  GdkPixbuf* targetimg=gdk_pixbuf_new(GDK_COLORSPACE_RGB, 0, 8, target.w, target.h);
  unsigned char* gdkpixels=gdk_pixbuf_get_pixels(targetimg);
  int gdkbpp=gdk_pixbuf_get_bits_per_sample(targetimg)*gdk_pixbuf_get_n_channels(targetimg)/8;
  int xbpp=screen->bits_per_pixel/8;
  // Copy out the part we selected
  int x, y;
  for(y=0; y<target.h; ++y)
  {
    for(x=0; x<target.w; ++x)
    {
      gdk_r(targetimg, x, y)=x_r(screen, target.x+x, target.y+y);
      gdk_g(targetimg, x, y)=x_g(screen, target.x+x, target.y+y);
      gdk_b(targetimg, x, y)=x_b(screen, target.x+x, target.y+y);
    }
  }
  XDestroyImage(screen);
  // Show the image to the user and ask for a name, allow to cancel, maybe we can set additional options too, like offsets and waiting
  GtkWidget* window=gtk_window_new(GTK_WINDOW_TOPLEVEL);
  if(opts->type==IMGCMD_CLICK)
  {
    gtk_window_set_title(GTK_WINDOW(window), "AutoX11 designer, click on");
  }
  else if(opts->type==IMGCMD_WAIT)
  {
    gtk_window_set_title(GTK_WINDOW(window), "AutoX11 designer, wait for");
  }
  gtk_window_set_transient_for(GTK_WINDOW(window), GTK_WINDOW(mainwindow));
  gtk_window_set_modal(GTK_WINDOW(window), 1);
  GtkWidget* vbox=gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
  gtk_container_add(GTK_CONTAINER(window), vbox);
  // Image
  GtkWidget* img=gtk_image_new_from_pixbuf(targetimg);
  gtk_box_pack_start(GTK_BOX(vbox), img, 0, 0, 10);
  // Target name
  GtkWidget* hbox=gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
  GtkWidget* label=gtk_label_new("Name of target:");
  gtk_box_pack_start(GTK_BOX(hbox), label, 0, 0, 0);
  GtkWidget* name_entry=gtk_entry_new();
  gtk_box_pack_start(GTK_BOX(hbox), name_entry, 1, 1, 0);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, 0, 0, 0);
  // Additional options, like offsets
  GtkWidget* opt_wait=0;
  GtkWidget* opt_xoffset=0;
  GtkWidget* opt_yoffset=0;
  GtkWidget* opt_button=0;
  if(opts->type==IMGCMD_CLICK)
  {
    // Wait (-w)
    opt_wait=gtk_check_button_new_with_label("Wait for target to appear");
    gtk_box_pack_start(GTK_BOX(vbox), opt_wait, 0, 0, 0);
    // X offset (-x n)
    hbox=gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
    gtk_box_pack_start(GTK_BOX(vbox), hbox, 0, 0, 0);
    gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new("X offset:"), 1, 1, 0);
    opt_xoffset=gtk_entry_new();
    char buf[16];
    sprintf(buf, "%i", gdk_pixbuf_get_width(targetimg)/2);
    gtk_entry_set_text(GTK_ENTRY(opt_xoffset), buf);
    gtk_box_pack_start(GTK_BOX(hbox), opt_xoffset, 0, 0, 0);
    // Y offset (-y n)
    hbox=gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
    gtk_box_pack_start(GTK_BOX(vbox), hbox, 0, 0, 0);
    gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new("Y offset:"), 1, 1, 0);
    opt_yoffset=gtk_entry_new();
    sprintf(buf, "%i", gdk_pixbuf_get_height(targetimg)/2);
    gtk_entry_set_text(GTK_ENTRY(opt_yoffset), buf);
    gtk_box_pack_start(GTK_BOX(hbox), opt_yoffset, 0, 0, 0);
    // Button (-b n)
    hbox=gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
    gtk_box_pack_start(GTK_BOX(vbox), hbox, 0, 0, 0);
    gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new("Button:"), 1, 1, 0);
    opt_button=gtk_combo_box_text_new_with_entry();
    gtk_combo_box_text_append(GTK_COMBO_BOX_TEXT(opt_button), 0, "1 (Left)");
    gtk_combo_box_text_append(GTK_COMBO_BOX_TEXT(opt_button), 0, "2 (Middle)");
    gtk_combo_box_text_append(GTK_COMBO_BOX_TEXT(opt_button), 0, "3 (Right)");
    gtk_combo_box_set_active(GTK_COMBO_BOX(opt_button), 0);
    gtk_box_pack_start(GTK_BOX(hbox), opt_button, 0, 0, 0);
  }
  // OK button
  hbox=gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
  GtkWidget* button=gtk_button_new_with_label("OK");
  void ok(GtkWidget* obj, void* d)
  {
    const char* name=gtk_entry_get_text(GTK_ENTRY(name_entry));
    char filename[strlen(name)+5];
    strcpy(filename, name);
    strcat(filename, ".png");
    // Ask about overwriting if it already exists
    if(!access(filename, F_OK))
    {
      GtkWidget* q=gtk_dialog_new_with_buttons("Overwrite?", GTK_WINDOW(window), GTK_DIALOG_MODAL, "_Ok", GTK_RESPONSE_ACCEPT, "_Cancel", GTK_RESPONSE_REJECT, (char*)0);
      GtkWidget* content=gtk_dialog_get_content_area(GTK_DIALOG(q));
      char msg[strlen("x already exists, do you wish to overwrite it?")+strlen(filename)];
      sprintf(msg, "%s already exists, do you wish to overwrite it?", filename);
      gtk_container_add(GTK_CONTAINER(content), gtk_label_new(msg));
      gtk_widget_show_all(q);
      if(gtk_dialog_run(GTK_DIALOG(q))!=GTK_RESPONSE_ACCEPT)
      {
        gtk_widget_destroy(q);
        return;
      }
      gtk_widget_destroy(q);
    }
// printf("Saving to '%s'\n", filename);
    gdk_pixbuf_save(targetimg, filename, "png", 0, (char*)0);
    GtkTextBuffer* buffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(srcview));
    GValue cursor=G_VALUE_INIT;
    g_value_init(&cursor, G_TYPE_INT);
    g_object_get_property(G_OBJECT(buffer), "cursor-position", &cursor);
    GtkTextIter iter;
    gtk_text_buffer_get_iter_at_offset(buffer, &iter, g_value_get_int(&cursor));
    char within=!(gtk_text_iter_starts_line(&iter)&&gtk_text_iter_ends_line(&iter));
    if(opts->type==IMGCMD_CLICK)
    {
      gtk_text_buffer_insert_at_cursor(buffer, "autox11 clickon ", 16);
      // Wait (-w)
      if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(opt_wait)))
      {
        gtk_text_buffer_insert_at_cursor(buffer, "-w ", 3);
      }
      // X offset (-x n)
      int offset=atoi(gtk_entry_get_text(GTK_ENTRY(opt_xoffset)));
      if(offset!=gdk_pixbuf_get_width(targetimg)/2)
      {
        const char* offset=gtk_entry_get_text(GTK_ENTRY(opt_xoffset));
        gtk_text_buffer_insert_at_cursor(buffer, "-x ", 3);
        gtk_text_buffer_insert_at_cursor(buffer, offset, strlen(offset));
        gtk_text_buffer_insert_at_cursor(buffer, " ", 1);
      }
      // Y offset (-y n)
      offset=atoi(gtk_entry_get_text(GTK_ENTRY(opt_yoffset)));
      if(offset!=gdk_pixbuf_get_height(targetimg)/2)
      {
        const char* offset=gtk_entry_get_text(GTK_ENTRY(opt_yoffset));
        gtk_text_buffer_insert_at_cursor(buffer, "-y ", 3);
        gtk_text_buffer_insert_at_cursor(buffer, offset, strlen(offset));
        gtk_text_buffer_insert_at_cursor(buffer, " ", 1);
      }
      // Button (-b n)
      const char* button=gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(opt_button));
      char* end;
      int button_num=strtol(button, &end, 0);
      int len=1;
      if(end==button){button="1"; button_num=1;}else{len=end-button;}
      if(button_num!=1)
      {
        gtk_text_buffer_insert_at_cursor(buffer, "-b ", 3);
        gtk_text_buffer_insert_at_cursor(buffer, button, len);
        gtk_text_buffer_insert_at_cursor(buffer, " ", 1);
      }
    }
    else if(opts->type==IMGCMD_WAIT){
      gtk_text_buffer_insert_at_cursor(buffer, "autox11 waitfor ", 16);
    }
    gtk_text_buffer_insert_at_cursor(buffer, filename, strlen(filename));
    // Don't add a newline if the command was inserted into an existing expression
    if(!within)
    {
      gtk_text_buffer_insert_at_cursor(buffer, "\n", 1);
    }
    gtk_widget_destroy(window);
  }
  g_signal_connect(name_entry, "activate", G_CALLBACK(ok), 0);
  g_signal_connect(button, "clicked", G_CALLBACK(ok), 0);
  gtk_box_pack_start(GTK_BOX(hbox), button, 1, 1, 0);
  // Cancel button
  button=gtk_button_new_with_label("Cancel");
  void cancel(GtkWidget* obj, void* d)
  {
    gtk_widget_destroy(window);
  }
  g_signal_connect(button, "clicked", G_CALLBACK(cancel), 0);
  gtk_box_pack_start(GTK_BOX(hbox), button, 1, 1, 0);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, 0, 0, 0);
  void abort(GtkWidget* obj, void* d)
  {
    g_object_unref(targetimg);
    window=0;
  }
  gtk_widget_show_all(window);
  g_signal_connect(window, "destroy", G_CALLBACK(abort), 0);
  while(window){gtk_main_iteration();}
}
